/*
	*** host interface:
	//
	host.setFaderLevel(int chn, double level);
		Tag type prefixes:
		s - string
		m - nested message
		b - byte
		bv - byte array
		w - word
		wv - word array
		d - double word
		dv - double word array
	host.sendCmsg(table message);
	// converts byte array of ascii codes to string.
	host.arrtostr(array, [start_index_from_1, [len]]);
	host.setDate(day, month, year, hours, minutes, seconds);
		value range:
		hours:   0..23
		minutes: 0..59
		seconds: 0..59
		day:     1..31
		month:   1..12
		year:    1990..3000
	//
	*** mod interface:
	//
	int mod.setChnText(int chn, string text, int flags);
	int mod.setChnKeyLed(int chn, int key, int style);
	int mod.setChnIcon(int chn, int icon, int flags);
	int mod.setMasterKeyLed(int key, int style);
	int mod.setGpioOut(int port, int index, int state);

	int mod.getChnKey(int chn, int key);
	int mod.getChnFader(int chn);
	int mod.getMasterKey(int key);
	int mod.getMasterRotary(int rotary);
	int mod.getGpioIn(int port, int index);
	string,int mod.getChnConfig(void);

	*** gui interface:
	// aux functions
	{"rgb",			j_rgb,			3},					// int gui.rgb(int r, int g, int b);
	{"rgba",		j_rgba,			4},					// int gui.rgba(int r, int g, int b, int a);

	// simple functions
	{"setBgImage", j_setBgImage,	1},		// bool gui.setBgImage(string src);
	{"setBgColor", j_setBgColor,	1},		// void gui.setBgColor(int color);

	// common window manipulation
	{"isWnd",			j_isWnd,				1},		// bool gui.isWnd(wnd window);

	// window parameters setting
	{"pictSetIndex",	j_pictSetIndex,		2},	// void gui.pictSetIndex(wnd picture, int index);
	{"meterSetValue", j_meterSetValue,	2},	// void gui.meterSetValue(wnd meter, int value);

	{"labelSetProps", j_labelSetProps,	2},	// void gui.labelSetProps(wnd label, table props);
	{"labelSetText",	j_labelSetText,		2},	// void gui.labelSetText(wnd label, string text);

	//list box
	{"listSetProps", j_listSetProps,		2},	// void gui.listSetProps(wnd list, table props);
	{"listInsert", j_listInsert,				2},	// void gui.listInsert(wnd list, string item [, int id [, int flags]]);
	{"listClear", j_listClear,					1},	// void gui.listClear(wnd list);
	{"listSetSelection", j_listSetSelection,	2},	// void gui.setListSelection(wnd list, int index);
	{"listAdvance", j_listAdvance, 2},						// void gui.listAdvance(wnd list, int step [, int flags]);

	{"listGetCount", j_listGetCount,		1},		// int gui.listGetCount(wnd list);
	{"listGetSelPos", j_listGetSelPos,	1},		// int gui.listGetSelPos(wnd list);
	{"listGetSelItem", j_listGetSelItem,1},		// int gui.listGetSelItem(wnd list);

	//for list
	//member functions
		{"setProps",	j_listSetPropsWnd,		1},
		{"insert",		j_listInsertWnd,			1},
		{"clear",			j_listClearWnd,				0},
		{"select",		j_listSelectWnd,			1},
		{"advance",		j_listAdvanceWnd,			1},

		{"goFirst",		j_listgoFirst,				0},
		{"goLast",		j_listgoLast,					0},
		{"goNext",		j_listgoNext,					0},
		{"goPrev",		j_listgoPrev,					0},
		{"goPos",			j_listgoPos,					1},
	//member properties
		{"id",				LST_ID,			JSPROP_PERMANENT, j_listGet, j_listSet},
		{"icon",			LST_ICON,		JSPROP_PERMANENT, j_listGet, j_listSet},
		{"flags",			LST_FLAGS,	JSPROP_PERMANENT, j_listGet, j_listSet},
		{"text",			LST_TEXT,		JSPROP_PERMANENT, j_listGet, j_listSet},

		{"count",			LST_COUNT,	JSPROP_PERMANENT|JSPROP_READONLY,	j_listGet, j_listSet},
		{"selection",	LST_SELECT,	JSPROP_PERMANENT,	j_listGet, j_listSet},

		{"selId",			LST_SELID,	JSPROP_PERMANENT|JSPROP_READONLY,	j_listGet, j_listSet},
		{"selTxt",		LST_SELTXT,	JSPROP_PERMANENT|JSPROP_READONLY,	j_listGet, j_listSet},

	// window creation
	{"newFrame", j_newFrame, 6},				// wnd gui.newFrame(wnd parent, int x, int y, int w, int h, string bg);
	{"newPicture", j_newPicture, 6},		// wnd gui.newPicture(wnd parent, int x, int y, int w, int h, string img);
	{"newLabel", j_newLabel, 6},				// wnd gui.newLabel(wnd parent, int x, int y, int w, int h, string img);
	{"newList", j_newList, 6},					// wnd gui.newList(wnd parent, int x, int y, int w, int h, string img);
	{"newMeter", j_newMeter, 6},				// wnd gui.newMeter(wnd parent, int x, int y, int w, int h, string img);

	*** log interface:
	void log.info(string text);
	void log.warning(string text);
	void log.error(string text);

	// -- GPIO command sample --
	host.sendCmsg({id:"NEST", mGPIO:{id:"WRNI", dLPID:10,
		bvPINN:[gpio.pin_start_pulse, gpio.pin_on_lamp, gpio.pin_off_lamp],
		bvVALL:[gpio.pulse, gpio.on, gpio.off]}});

*/

// ------------------------- * global variables * -------------------------
var txtChn=[];
// UI timeouts
var confModeTO=null, ipPageTO=null, timerPageTO=null, showRestoreTO=null;

function makeTags()
{
	var n;
	for(n=1; n<=64; n++)
		txtChn[n]=host.sprintf("mCH%02d", n);
}
makeTags();

// ------------------------- * persistent configuration * -------------------------

/*
** readConfiguration
*/
function readConfiguration()
{
	var n, m;

/* convert old style VMIXER configuration with 4294967295 instead of -1 */
	for(n=1; n<=8; n++)
		for(m=1; m<=5; m++)
	{
		if(host.cfgGetString("VMIX.VM0"+n+".IN0"+m+".SRCI", "")=="4294967295")
			host.cfgSetInteger("VMIX.VM0"+n+".IN0"+m+".SRCI", -1);
	}

/* read configuration values */
	cfg_surroundMode =host.cfgGetString ('scr.surroundMode',  cfg_surroundMode?'yes':'no')=='yes';
	right_mtr_tag    =cfg_surroundMode?'mMET2':'mMET4';
	cfg_clock24hour  =host.cfgGetString ('scr.clock24hour',   cfg_clock24hour ?'yes':'no')=='yes';
	cfg_countdownfrom=host.cfgGetInteger('scr.countdownFrom', cfg_countdownfrom);
	cfg_countdownfrom=clamp(cfg_countdownfrom, 0, timerMaxValue);
/* write values back */
	host.cfgSetString ('scr.surroundMode',  cfg_surroundMode?'yes':'no');
	host.cfgSetString ('scr.clock24hour',   cfg_clock24hour ?'yes':'no');
	host.cfgSetInteger('scr.countdownFrom', cfg_countdownfrom);

/* read VMIXER configuration */
	vmixer=[];
	for(n=1; n<=8; n++)
	{
		vmixer["VM0"+n]={};
		vmixer["VM0"+n].LPID=host.cfgGetInteger("VMIX.VM0"+n+".LPID", 0);
		vmixer["VM0"+n].MSGN=host.cfgGetInteger("VMIX.VM0"+n+".MSGN", 0);
		for(m=1; m<=5; m++)
		{
			vmixer["VM0"+n]["IN0"+m]=[];
			vmixer["VM0"+n]["IN0"+m].RAMP=host.cfgGetInteger("VMIX.VM0"+n+".IN0"+m+".RAMP", 0);
			vmixer["VM0"+n]["IN0"+m].SRCI=host.cfgGetInteger("VMIX.VM0"+n+".IN0"+m+".SRCI", -1);
			vmixer["VM0"+n]["IN0"+m].FAGN=host.cfgGetInteger("VMIX.VM0"+n+".IN0"+m+".FAGN", 0);
			vmixer["VM0"+n]["IN0"+m].TMUP=host.cfgGetInteger("VMIX.VM0"+n+".IN0"+m+".TMUP", 5);
			vmixer["VM0"+n]["IN0"+m].TMDN=host.cfgGetInteger("VMIX.VM0"+n+".IN0"+m+".TMDN", 5);
		}
	}
}

// ------------------------- * Meters * -------------------------
const mtrNumBars=91;
const mtrPoints=[-500, 0, -100, 81, 0, 91];

function createStereoMeter(frm, guiId, engId)
{
	var mp, ml, mr;

	mp=gui.newPicture(frm, 41, 44, 52, 19, "img/mtr_peak.png");
	ml=gui.newMeter(frm, 27, 74, 25, 364, "img/mbar_l.png"); ml.setBars(mtrNumBars); ml.setPoints(mtrPoints);
	mr=gui.newMeter(frm, 84, 74, 25, 364, "img/mbar_r.png"); mr.setBars(mtrNumBars); mr.setPoints(mtrPoints);
	gui.createMeter(guiId, 4, mp, [ml, mr]);
	gui.linkMeter(guiId, engId);
	host.requestMeter(engId, true);
}

function createSurroundMeter(frm, guiId, engId)
{
	var mposx=[27, 50, 108, 131, 154, 177, 200, 223];
	var mp, barr=[];
	var n;

	mp=gui.newPicture(frm, 50, 42, 52, 19, "img/mtr_peak.png");
	for(n=0; n<8; n++)
	{
		barr[n]=gui.newMeter(frm, mposx[n], 75, 19, 364, "img/mbar_8ch.png");
		barr[n].setBars(mtrNumBars); barr[n].setPoints(mtrPoints);
	}
	gui.createMeter(guiId, 4, mp, barr);
	gui.linkMeter(guiId, engId);
	host.requestMeter(engId, true);
}

function meterSetConstants(eId, tm)
{
	var tag=host.sprintf("mM%03u", eId);
	host.sendNestCmsg(tag, {id:'WRIN', dTM01: tm[0], dTM02: tm[1]});
}

function createMultiMeter(frm, x, guiId, engId, type)
{
	var mscale, mp, ml, mr;
	var tm;

	switch(type)
	{
	default:
	case "VU":
		mscale=gui.newFrame(frm, x, 54, 102, 400, "img/mtr_scale_vu.png");
		mp=gui.newPicture(frm, x+18, 54-9-7, 67, 7, "img/mtr_ovr.png");
		ml=gui.newMeterEx(mscale, 18, 2, 17, 396, "img/mbar_ex.png"); ml.setPoints([-600, 0, -500, 9, -100, 89, 0, 99]); ml.setCPoint(68);
		mr=gui.newMeterEx(mscale, 68, 2, 17, 396, "img/mbar_ex.png"); mr.setPoints([-600, 0, -500, 9, -100, 89, 0, 99]); mr.setCPoint(68);
		tm=[65000, 65000];
		break;
	case "EBUD":
		mscale=gui.newFrame(frm, x+13, 54, 77, 400, "img/mtr_scale_ebud.png");
		mp=gui.newPicture(frm, x+18, 54-9-7, 67, 7, "img/mtr_ovr.png");
		ml=gui.newMeterEx(mscale, 18-13, 2, 17, 396, "img/mbar_ex.png"); ml.setPoints([-600, 0, -400, 9, -300, 19, -200, 39, -50, 84, 0, 99]); ml.setCPoint(71);
		mr=gui.newMeterEx(mscale, 68-13, 2, 17, 396, "img/mbar_ex.png"); mr.setPoints([-600, 0, -400, 9, -300, 19, -200, 39, -50, 84, 0, 99]); mr.setCPoint(71);
		tm=[1255, 738000];
		break;
	case "DIN":
		mscale=gui.newFrame(frm, x+13, 54, 76, 400, "img/mtr_scale_din.png");
		mp=gui.newPicture(frm, x+18, 54-9-7, 67, 7, "img/mtr_ovr.png");
		ml=gui.newMeterEx(mscale, 18-13, 2, 17, 396, "img/mbar_ex.png"); ml.setPoints([-600-90, 0, -400-90, 9, -300-90, 19, -50-90, 69, 50-90, 99]); ml.setCPoint(84);
		mr=gui.newMeterEx(mscale, 68-13, 2, 17, 396, "img/mbar_ex.png"); mr.setPoints([-600-90, 0, -400-90, 9, -300-90, 19, -50-90, 69, 50-90, 99]); mr.setCPoint(84);
		tm=[1700, 650000];
		break;
	case "NORDIC":
		mscale=gui.newFrame(frm, x+13-1, 54+2-18, 78, 419, "img/mtr_scale_nord.png");
		mp=gui.newPicture(frm, x+18, 54-9-7-10, 67, 7, "img/mtr_ovr.png");
		ml=gui.newMeterEx(mscale, 18-13, 2, 17, 416, "img/mbar_ex.png"); ml.setPoints([-420-180, 0, -360-180, 8, 120-180, 104]); ml.setCPoint(92);
		mr=gui.newMeterEx(mscale, 69-13, 2, 17, 416, "img/mbar_ex.png"); mr.setPoints([-420-180, 0, -360-180, 8, 120-180, 104]); mr.setCPoint(92);
		tm=[1700, 650000];
		break;
	case "BBC":
		mscale=gui.newFrame(frm, x+13-1, 54, 78, 398, "img/mtr_scale_bbc.png");
		mp=gui.newPicture(frm, x+18, 54-9-7, 67, 7, "img/mtr_ovr.png");
		ml=gui.newMeterEx(mscale, 18-13, 2, 17, 396, "img/mbar_ex.png"); ml.setPoints([-200-180, 0, -120-180, 15, 120-180, 99]); ml.setCPoint(87);
		mr=gui.newMeterEx(mscale, 69-13, 2, 17, 396, "img/mbar_ex.png"); mr.setPoints([-200-180, 0, -120-180, 15, 120-180, 99]); mr.setCPoint(87);
		tm=[2483, 1013000];
		break;
	}
	gui.createMeter(guiId, 4, mp, [ml, mr]);
	gui.linkMeter(guiId, engId);
	host.requestMeter(engId, true);
	meterSetConstants(engId+100, tm);

	return {parent: frm, x: x, guiId: guiId, engId: engId, type: type, scale: mscale, peak: mp};
}

function reCreateMultiMeter(meter, newType)
{
	if(newType==meter.type) return meter;
// remove old meter

	host.requestMeter(meter.engId, false);
	meter.scale.destroy();
	meter.peak.destroy();
	return createMultiMeter(meter.parent, meter.x, meter.guiId, meter.engId, newType);
}

function changeMeterBallistics(newType)
{
	var n;

	if(!(newType in mtrTypes)) newType=0;
	host.cfgSetInteger('meter.ballistics', newType);

	for(n in mtr_meter)
		mtr_meter[n]=reCreateMultiMeter(mtr_meter[n], mtrTypes[newType]);
}

/*
** makeMeters
**
** creates 4 main meters
*/
function makeMeters(lblock, rblock)
{
	var mframe, ballistics;

	if(cfg_surroundMode)
	{
	// PGM1 5.1+Stereo Meter
		mframe=gui.newFrame(lblock, 0, 0, 285, 531, "img/meter_8ch.png");
		createSurroundMeter(mframe, mtrGUI.pgm1, mtrEng.pgm1);
		mtr_label[1]=GUI_createMeterLabel(mframe, 95, 459, "PGM 1");

	// PGM2/CR/Preview 5.1+Stereo Meter
		mframe=gui.newFrame(rblock, 0, 0, 285, 531, "img/meter_8ch.png");
		createSurroundMeter(mframe, mtrGUI.pgm2, mtrEng.pgm2);
		mtr_label[2]=GUI_createMeterLabel(mframe, 95, 459, "PGM 2");

	// PGM3/PGM4 are not used, assign dummy objects to them
		mtr_label[3]=mtr_label[4]={activate:function(on){},setText:function(t){}};

	// Rightmost label is special
		mtr_label_right=mtr_label[2];
	}
	else
	{
		ballistics=host.cfgGetInteger('meter.ballistics', 0);
		if(!(ballistics in mtrTypes)) ballistics=0;
		

		mframe=gui.newFrame(lblock, 6, 7, 272, 518, "img/mtrbg_stpgm.png");
		mtr_label[1]=GUI_createMeterLabel(mframe, 34, 460, "PGM 1");
		mtr_meter[1]=createMultiMeter(mframe, 14, mtrGUI.pgm1, mtrEng.pgm1, mtrTypes[ballistics]);

		mtr_label[2]=GUI_createMeterLabel(mframe, 34+128, 460, "PGM 2");
		mtr_meter[2]=createMultiMeter(mframe, 142, mtrGUI.pgm2, mtrEng.pgm2, mtrTypes[ballistics]);

		mframe=gui.newFrame(rblock, 6, 7, 272, 518, "img/mtrbg_stpgm.png");
		mtr_label[3]=GUI_createMeterLabel(mframe, 34, 460, "PGM 3");
		mtr_meter[3]=createMultiMeter(mframe, 14, mtrGUI.pgm3, mtrEng.pgm3, mtrTypes[ballistics]);

		mtr_label[4]=GUI_createMeterLabel(mframe, 34+128, 460, "PGM 4");
		mtr_meter[4]=createMultiMeter(mframe, 142, mtrGUI.pgm4, mtrEng.pgm4, mtrTypes[ballistics]);

	// Rightmost label is special
		mtr_label_right=mtr_label[4];
	}
}

// ------------- * create all graphical objects * -------------------

function initGUI()
{
	var i, j;
	var lblock, rblock;

	log.info_f("Element %s (%s) %s is starting...", version, version_date, host.getIdentity());
	readConfiguration();

	cur_show_name = "-- NO SHOW --";	// id of current show, initially not set
	cur_show_id = null;	// id of current show, initially not set
	cur_show_loading = false;	// true if some profile is loading, false if not loading
	pview_interlock = false;		// preview interlock mode
	recmode_enabled = false;    // Record Mode assumed not enabled
	recmode_rcdrlpid = 0;       // Recorder's LPID for Record Mode
	recmode_active = false;     // Record Mode is initially off

	pages = [];	// stores all references to pages && their controls
	//register of all functions that are listening on messages
	//it is updated on page open and close
	pages.rxHandlersGlobal = [];
	pages.rxHandlersCh = [];
	pages.open=null;
	mstEnable=
	{
		mask    : 0,
	// Channel Options
		chn_src : true, //  1 Source
		chn_eq  : true, //  2 EQ&Dynamics
		chn_pan : true, //  3 PAN&Mode
		chn_aux : true, //  4 AUX / SENDS
		chn_feed: true, //  5 Feed to Source
		opt_mon : true, //  6 Monitor Options
		opt_mtr : true, //  7 Meter Options
		opt_snd : true, //  8 Sends & Returns
		opt_fdr : true, //  9 Fader Options
		delay   : true, // 10 Delay (all keys 'dump', 'ext', 'pause' and 'engage' together)
		timer   : true, // 11 Timer (all keys 'options', 'reset', 'freeze' and 'start/stop' together)
		ttmon2  : true, // 12 Talk to Monitor 2
		prev_hp : true, // 13 Preview to HP
		link    : true, // 14 Link
		numpad  : true, // 15 Numeric Keypad
		crmo_pgm: true, // 16 Program Monitor 1 (all keys '1', '2', '3' and '4' together)
		crmo_aux: true, // 17 AUX SENDS Monitor 1 (all keys 'A', 'B', 'C' and 'D' together)
		crmo_ext: true, // 18 External Monitor 1 (both keys '1' and '2' together)
		crhp_pgm: true, // 19 Program Headphones (all keys '1', '2', '3' and '4' together)
		crhp_aux: true, // 20 AUX SENDS Headphones (all keys 'A', 'B', 'C' and 'D' together)
		crhp_ext: true, // 21 External Headphones (both keys '1' and '2' together)
		src_mon2: true, // 22 Monitor2 source selection
		vol_mon1: true, // 23 Volume Control of Monitor1 (CRMO)
		vol_mon2: true, // 24 Volume Control of Monitor2 (STMO)
		vol_crhp: true, // 25 Volume Control of Headphones (CRHP)
		vol_prev: true, // 26 Volume Control of Preview
		stmo_pgm: true, // 27 Program Monitor 2
		stmo_aux: true, // 28 AUX SENDS Monitor 2
		stmo_ext: true, // 29 External Monitor 2
		update: function(mask)
		{
			this.mask=mask; // Cache Mask
			this.chn_src =((mask&(1<<0))!=0); // 1 Source
			this.chn_eq  =((mask&(1<<1))!=0); // 2 EQ&Dynamics
			this.chn_pan =((mask&(1<<2))!=0); // 3 PAN&Mode
			this.chn_aux =((mask&(1<<3))!=0); // 4 AUX / SENDS
			this.chn_feed=((mask&(1<<4))!=0); // 5 Feed to Source

			this.opt_mon=((mask&(1<<5))!=0); // 6 Monitor Options
			this.opt_mtr=((mask&(1<<6))!=0); // 7 Meter Options
			this.opt_snd=((mask&(1<<7))!=0); // 8 Sends & Returns
			this.opt_fdr=((mask&(1<<8))!=0); // 9 Fader Options

			this.delay  =((mask&(1<<9))!=0); // 10 Delay (all keys 'dump', 'ext', 'pause' and 'engage' together)
			this.timer  =((mask&(1<<10))!=0); // 11 Timer (all keys 'options', 'reset', 'freeze' and 'start/stop' together)
			this.ttmon2 =((mask&(1<<11))!=0); // 12 Talk to Monitor 2
			this.prev_hp=((mask&(1<<12))!=0); // 13 Preview to HP
			this.link   =((mask&(1<<13))!=0); // 14 Link
			this.numpad =((mask&(1<<14))!=0); // 15 Numeric Keypad and/or Scrub Wheel/Nav Keys together

			this.crmo_pgm=((mask&(1<<15))!=0); // 16 Program Monitor 1 (all keys '1', '2', '3' and '4' together)
			this.crmo_aux=((mask&(1<<16))!=0); // 17 AUX SENDS Monitor 1 (all keys 'A', 'B', 'C' and 'D' together)
			this.crmo_ext=((mask&(1<<17))!=0); // 18 External Monitor 1 (both keys '1' and '2' together)

			this.crhp_pgm=((mask&(1<<18))!=0); // 19 Program Headphones (all keys '1', '2', '3' and '4' together)
			this.crhp_aux=((mask&(1<<19))!=0); // 20 AUX SENDS Headphones (all keys 'A', 'B', 'C' and 'D' together)
			this.crhp_ext=((mask&(1<<20))!=0); // 21 External Headphones (both keys '1' and '2' together)

			this.src_mon2=((mask&(1<<21))!=0); // 22 Monitor2 source selection
			this.vol_mon1=((mask&(1<<22))!=0); // 23 Volume Control of Monitor1 (CRMO)
			this.vol_mon2=((mask&(1<<23))!=0); // 24 Volume Control of Monitor2 (STMO)
			this.vol_crhp=((mask&(1<<24))!=0); // 25 Volume Control of Headphones (CRHP)
			this.vol_prev=((mask&(1<<25))!=0); // 26 Volume Control of Preview

			this.stmo_pgm=((mask&(1<<26))!=0); // 27 Program Monitor 2 (all keys '1', '2', '3' and '4' together)
			this.stmo_aux=((mask&(1<<27))!=0); // 28 AUX SENDS Monitor 2 (all keys 'A', 'B', 'C' and 'D' together)
			this.stmo_ext=((mask&(1<<28))!=0); // 29 External Monitor 2 (both keys '1' and '2' together)
		}
	};

	vmixer_loaded = 0;

// init channel table
	chnInitTable();

	for (i = 1; i <= 64; i++){
		pages.rxHandlersCh[i] = {};
	}

	// External Sources status
	ext1_src_status = 128;
	ext2_src_status = 128;

	mon_link=false;
	mon_mon1src=0;
	mon_hpsrc=0;
	upTimerObj=null;	// will be created in main page new Timer();
	dnTimerObj=null;	// will be created in main page new Timer();
	// make jog wheel less sensitive
	jogDelta  = 0;
	crmo_lpid = 0;
	stmo_lpid = 0;
	prdl_lpid = 0;
	utmr_lpid = 0;
	dtmr_lpid = 0;

	gui.init(1024, 768);
	gui.setBgImage("img/bg.png");
	bgframe=gui.newFrame(null, 0, 0, 1024, 768, "");
	bgframe.hide();

	mainframe=gui.newFrame(bgframe, 288,  13, 463, 527, "img/frm_main.png");

	// --- meter blocks ---
	lblock=gui.newFrame(bgframe,  13, 12, 273, 550, "");
	rblock=gui.newFrame(bgframe, 744, 12, 273, 550, "");
	makeMeters(lblock, rblock);

	Gui_CreateIndiSection();

	mode_txt=cfg_surroundMode?mode_txt_surround:mode_txt_stereo;

	// create all pages with graphical elements in memory (initially hidden)
	createPages();

	bgframe.show();
}

function onMasterRotaryEvent(index, delta, newValue)
{
	switch(index)
	{
	case knobs.MstJog: // Jog wheel
		jogDelta+=delta;
		if(jogDelta>4 || jogDelta<-4)
		{
			sknobsOnRot('jog', (jogDelta>0)?1:-1);
			jogDelta=0;
		}
		break;
	case knobs.MstMon2: // Monitor 2 gain
	case knobs.CmbMon2:
		if(pages.open.id=="monsrc" && pages.open.channel==chn_mon2)
			sknobsOnRot('jog', delta);
		else if(mstEnable.vol_mon2)
			adjustMonGain("wSTMG", "mon2", delta, 35);
		break;
	case knobs.CmbPview:
		if(pages.open.id=="monsrc" && pages.open.channel==chn_hp)
		{
			sknobsOnRot('jog', delta);
			break;
		}
		/* fall through to change preview gain */
	case knobs.MstPview: // Preview gain
		if(mstEnable.vol_prev) adjustMonGain("wPVWG", "pvw", delta, 35);
		break;
	case knobs.MstMon1: // Monitor 1 gain
	case knobs.CmbMon1:
		if(mstEnable.vol_mon1) adjustMonGain("wCRMG", "mon1", delta, 10);
		break;
	case knobs.MstHP: // HP gain
	case knobs.CmbHP:
		if(mstEnable.vol_crhp) adjustMonGain("wCRHG", "hp", delta, 10);
		break;
	default:
		if(index>=knobs.MstSknob1 && index<=knobs.MstSknob6)
			sknobsOnRot(index-knobs.MstSknob1, delta);
		else if(index>=knobs.CmbSknob1 && index<=knobs.CmbSknob3)
			sknobsOnRot(6+(index-knobs.CmbSknob1), delta);
		break;
	}
}

// capture
function confStart()
{
	confModeTO=null;
	activatePage(pages.help, 1);
}
// ip page
function ipPageStart()
{
	ipPageTO=null;
	activatePage(pages.ip, null);
}
// timer page
function timerPageStart()
{
	timerPageTO=null;
	activatePage(pages.timer, null);
}
// restore current show
function showRestoreStart()
{
	showRestoreTO=null;
// reload current show
	host.sendNestCmsg('mSHOW', {id:"WRIN", dSHWI:cur_show_id});
}
var comboMtrxUsed=false;

function onMasterKeyDown(key)
{
	var msgr;

	switch(key)
	{
	//navigation keys
	case keys.MstEnter:
		if(gui_indi.numpadEvent(13, true)) break;
		sknobsOnDn('enter');
		break;
	case keys.MstUp:
		sknobsOnRot('updn', 1);
		break;
	case keys.MstDown:
		sknobsOnRot('updn', -1);
		break;
	case keys.MstLeft:
		if(gui_indi.numpadEvent(12, true)) break;
		sknobsOnRot('ltrt', -1);
		break;
	case keys.MstRight:
		sknobsOnRot('ltrt', 1);
		break;
	case keys.MstHelp:
	// configuration mode
		confModeTO=host.resetTimeout(confModeTO, 5000, confStart);
		break;

	//global options
	case keys.MstProfile:
	case keys.CmbProfile:
	// show page on short push, restore current show on push and hold
		showRestoreTO=host.resetTimeout(showRestoreTO, 5000, showRestoreStart);
		break;
	case keys.MstMonOpt:
		if(!mstEnable.opt_mon) break;
		activatePage(pages.monopt, null);
		break;
	case keys.MstMetOpt:
		if(!mstEnable.opt_mtr) break;
		activatePage(pages.meters, null);
		break;
	case keys.MstSendRet:
		if(!mstEnable.opt_snd) break;
		if (pages.open == pages.sends || pages.open == pages.returns)
			//open returns if sends is open, or close returns if returns is already open
			activatePage(pages.returns, null);
		else
			activatePage(pages.sends, null);
		break;
	case keys.MstFaderOpt:
		ipPageTO=host.clearTimeout(ipPageTO);
		if(pages.open==pages.ip) // if already open, then close immediately
			activatePage(pages.ip, null);
		// Fader Screen Removed
		//else if(pages.open==pages.faders) // if already open, then close immediately
		//	activatePage(pages.faders, null);
		else // ip screen after 5 sec
			ipPageTO=host.setTimeout(5000, ipPageStart);
		break;

	//channel options
	case keys.MstChnSrc:
		if(!mstEnable.chn_src) break;
		chSwitchToPage(pages.chsrc);
		break;
	case keys.MstChnEq:
		if(!mstEnable.chn_eq) break;
		if ((pages.open == pages.cheq || pages.open == pages.chmicproc )
		    && 'channel' in pages.open
		    && ch_data[pages.open.channel].srct != srct.line && ch_data[pages.open.channel].srct != srct.computer)
			// open micproc or go back to main
			chSwitchToPage(pages.chmicproc);
		else
			chSwitchToPage(pages.cheq);
		break;
	case keys.MstChnPan:
		if(!mstEnable.chn_pan) break;
		chSwitchToPage(pages.chpan);
		break;
	case keys.MstChnAux:
		if(!mstEnable.chn_aux) break;
		chSwitchToPage(pages.chaux);
		break;
	case keys.MstChnM1:
		if(!mstEnable.chn_feed) break;
		chSwitchToPage(pages.chm1);
		break;

	// delay
	case keys.MstDump:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 8, gpio.on);
		break;
	case keys.MstExit:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 7, gpio.on);
		break;
	case keys.MstPause:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 6, gpio.on);
		break;
	case keys.MstEngage:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 5, gpio.on);
		break;

// timer
	case keys.MstTimerOpt:
	case keys.CmbTimerOpt:
		if(!mstEnable.timer) break;
		timerPageTO=host.resetTimeout(timerPageTO, 5000, timerPageStart);
		break;
	case keys.MstTimerReset:
	case keys.CmbTimerReset:
		if(!mstEnable.timer) break;
		upTimerObj.reset();
		break;
	case keys.MstTimerPause:
	case keys.CmbTimerPause:
		if(!mstEnable.timer) break;
		upTimerObj.freeze();
		break;
	case keys.MstTimerStart:
	case keys.CmbTimerStart:
		if(!mstEnable.timer) break;
		upTimerObj.toggleStart();
		break;

	case keys.MstRecord:
	case keys.CmbRecord:
		if(recmode_active)
			deactivateRecordMode();
		else
			activateRecordMode();
		break;

// Monitor Section
// Monitor2 (STMO)
	case keys.CmbMon2Knob:
		if(mod.getMasterKey(keys.CmbMonOpt))
		{
		// Extra Meters Page activation
			comboMtrxUsed=true;
			activatePage(pages.mtrx, null);
			break;
		}
	/* fall through */
	case keys.MstMon2Knob:
		if(pages.open.id=="monsrc" && pages.open.channel==chn_mon2)
			sknobsOnDn('enter');
		if(mstEnable.src_mon2) activatePage(pages.monsrc, chn_mon2);
		break;
	case keys.CmbPviewKnob:
		if(pages.open.id=="monsrc" && pages.open.channel==chn_hp)
			sknobsOnDn('enter');
		if(!mon_link) activatePage(pages.monsrc, chn_hp);
		break;
	case keys.MstTalkToMon2:
	case keys.CmbTalkToMon2:
		if(!mstEnable.ttmon2) break;
		host.sendNestCmsg('mTALK', {id:"WRIN", bSTUD:1});
		break;
	case keys.MstPvwToHp:
	case keys.CmbPvwToHp:
		if(!mstEnable.prev_hp) break;
		msgr=host.sendNestCmsgRq('mCRHP', {id:"READ", bPVMO:0});
		if(msgr!=null && ('bPVMO' in msgr))
			host.sendNestCmsg('mCRHP', {id:"WRIN", bPVMO:(msgr.bPVMO%3+1)});
		break;
	case keys.MstLink:
		if(!mstEnable.link) break;
		msgr=host.sendNestCmsgRq('mCRHP', {id:"READ", bINDP:0});
		if(msgr!=null && ('bINDP' in msgr))
			host.sendNestCmsg('mCRHP', {id:"WRIN", bINDP:(msgr.bINDP==1?2:1)});
		break;
// Monitor1 (CRMON)
	case keys.MstMon1Pgm1:
	case keys.MstMon1Pgm2:
	case keys.MstMon1Pgm3:
	case keys.MstMon1Pgm4:
	case keys.CmbMonPgm1:
	case keys.CmbMonPgm2:
	case keys.CmbMonPgm3:
	case keys.CmbMonPgm4:
		if(!mstEnable.crmo_pgm) break;
		onMon1KeyDown(key);
		break;
	case keys.MstMon1AuxA:
	case keys.MstMon1AuxB:
	case keys.MstMon1AuxC:
	case keys.MstMon1AuxD:
		if(!mstEnable.crmo_aux) break;
		onMon1KeyDown(key);
		break;
	case keys.MstMon1Ext1:
	case keys.MstMon1Ext2:
	case keys.CmbMonExt1:
	case keys.CmbMonExt2:
		if(!mstEnable.crmo_ext) break;
		onMonExtKeyDown(key);
		break;

	//Headphones (CRHP)
	case keys.MstHpPgm1:
	case keys.MstHpPgm2:
	case keys.MstHpPgm3:
	case keys.MstHpPgm4:
		if(!mstEnable.crhp_pgm) break;
		onHpKeyDown(key);
		break;
	case keys.MstHpAuxA:
	case keys.MstHpAuxB:
	case keys.MstHpAuxC:
	case keys.MstHpAuxD:
		if(!mstEnable.crhp_aux) break;
		onHpKeyDown(key);
		break;
	case keys.MstHpExt1:
	case keys.MstHpExt2:
		if(!mstEnable.crhp_ext) break;
		onMonExtKeyDown(key);
		break;

// cases that doesn't map to one single key
	default:
		if(key>=keys.MstSknob1 && key<=keys.MstSknob6)
			sknobsOnDn(key-keys.MstSknob1);
		else if(key>=keys.CmbSknob1 && key<=keys.CmbSknob3)
			sknobsOnDn(6+(key-keys.CmbSknob1));
		else if(key>=keys.MstNum0 && key<=keys.MstNumNum)
			onDTMFkeyDown(key-keys.MstNum0);
		else if(key>=keys.CmbNum0 && key<=keys.CmbNumNum)
			onDTMFkeyDown(key-keys.CmbNum0);
		break;
	}
}

function onMasterKeyUp(key)
{
	switch (key)
	{
	case keys.MstProfile:
	case keys.CmbProfile:
	// show page on short push, restore current show on push and hold
		if(showRestoreTO!=null)
		{
			showRestoreTO=host.clearTimeout(showRestoreTO);
			activatePage(pages.show, null);
		}
		break;
	case keys.MstHelp:
		// configuration mode
		if(confModeTO!=null)
		{
			confModeTO=host.clearTimeout(confModeTO);
			activatePage(pages.help, null);
		}
		break;
	case keys.MstFaderOpt:
	// ip screen on push and hold, faders screen on short push
		if(ipPageTO!=null)
		{
		// short push
			ipPageTO=host.clearTimeout(ipPageTO);
		// Fader Screen Removed
		//	if(mstEnable.opt_fdr) activatePage(pages.faders, null);
		}
		break;
	case keys.CmbMonOpt:
		if(!mstEnable.opt_mon) break;
		if(comboMtrxUsed)
		{
			comboMtrxUsed=false;
			break;
		}
		if(pages.open==pages.monopt || pages.open==pages.monopt2)
			activatePage(pages.monopt2, null);
		else
			activatePage(pages.monopt, null);
		break;

	// delay
	case keys.MstDump:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 8, gpio.off);
		break;
	case keys.MstExit:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 7, gpio.off);
		break;
	case keys.MstPause:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 6, gpio.off);
		break;
	case keys.MstEngage:
		if(!mstEnable.delay) break;
		sendGPIOcommand(prdl_lpid, 5, gpio.off);
		break;

	// clock timer screen
	case keys.MstTimerOpt:
	case keys.CmbTimerOpt:
		if(!mstEnable.timer) break;
		if(timerPageTO==null) return; // timer fired, skip this event

		timerPageTO=host.clearTimeout(timerPageTO);

		if(pages.open==pages.timer)
			activatePage(pages.timer, null);	// close timer options screen
		else
			mainShowTimerCtrl(key==keys.CmbTimerOpt);
		break;
	case keys.MstTimerPause:
	case keys.CmbTimerPause:
		if(!mstEnable.timer) break;
		upTimerObj.unfreeze(); // TODO: need to check if other key is pressed
		break;

	//monitor section
	//Monitor2
	case keys.MstTalkToMon2:
	case keys.CmbTalkToMon2:
		if(!mstEnable.ttmon2) break;
		host.sendNestCmsg('mTALK', {id:"WRIN", bSTUD:0});
		break;
	//Mon1
	case keys.MstMon1Ext1:
	case keys.MstMon1Ext2:
	case keys.CmbMonExt1:
	case keys.CmbMonExt2:
		if(!mstEnable.crmo_ext) break;
		onMonExtKeyUp(key);
		break;
	//HP
	case keys.MstHpExt1:
	case keys.MstHpExt2:
		if(!mstEnable.crhp_ext) break;
		onMonExtKeyUp(key);
		break;

// cases that doesn't map to one single key
	default:
		if(key>=keys.MstSknob1 && key<=keys.MstSknob6)
			sknobsOnUp(key-keys.MstSknob1);
		else if(key>=keys.CmbSknob1 && key<=keys.CmbSknob3)
			sknobsOnUp(6+(key-keys.CmbSknob1));
		else if(key>=keys.MstNum0 && key<=keys.MstNumNum)
			onDTMFkeyUp(key-keys.MstNum0);
		else if(key>=keys.CmbNum0 && key<=keys.CmbNumNum)
			onDTMFkeyUp(key-keys.CmbNum0);
		break;
	}
}

// ------------------------- * Keypad Functions * -------------------------
var dtmfAsteriskTimer=null;
function dtmfAsteriskAction()
{
	var n, v;

	dtmfAsteriskTimer=null;
	for(n=0, v=0; n<12; n++)
		if(mod.getMasterKey(keys.MstNum0+n) || mod.getMasterKey(keys.CmbNum0+n))
			v|=(1<<n);

	switch(v)
	{
	case (1<<10)|(1<< 2): // "*" & "2" - capture mode
		mod.enterCapture();
		break;
	case (1<<10)|(1<<11): // "*" & "#" - help screen
		activatePage(pages.help, null);
		break;
	case (1<<10)|(1<<2)|(1<<11): // "*" & "2" & "#" - test mode
		if(testAttached) mod.testBegin();
		break;
	case (1<<10)|(1<<4)|(1<<7): // "*" & "4" & "7" - IP screen
		activatePage(pages.ip, null);
		break;
	}
}

function onDTMFkeyDownEx(key)
{
	if(!mstEnable.numpad) return;

/* first send numpad event to screens */
	if(onNumpadDown(key)) return;
/* if not utilized yet - send to dialpad */
	if(gui_indi.numpadEvent(key, true)) return;
}

function onDTMFkeyDown(key)
{
	if(key==10) // "*"
	{
		dtmfAsteriskTimer=host.resetTimeout(dtmfAsteriskTimer, 5000, dtmfAsteriskAction);
		return;
	}
	if(dtmfAsteriskTimer) return;
	onDTMFkeyDownEx(key);
}

function onDTMFkeyUp(key)
{
	if(dtmfAsteriskTimer)
	{
	// timer in progress
		if(key==10) // "*"
		{
			dtmfAsteriskTimer=host.clearTimeout(dtmfAsteriskTimer);
			onDTMFkeyDownEx(key);
		}
		else
			return;
	}

	if(!mstEnable.numpad) return;
	if(gui_indi.numpadEvent(key, false)) return;
}

/*
** onConfigChange
**
** called when host configuration changes. Includes:
**  * fader channel configuration
**  * phone channel configuration
**
** updates internal script config and sends message to acl
** CONF
**   CMSG - string - current channel mask
**   PHON - message - phone channel data
**   SURR - byte   - signal mode - surround or stereo
*/
function onConfigChange()
{
	var ch_config, ph_config;
	var conf_msg={id:'WRNI'};

// get configuration
	ch_config=mod.getChnConfig();
	ph_config=mod.getPhoneConfig();

// fader channels
	conf_msg.sCMSK=chnInitMask(ch_config);
// phone channels
	conf_msg.mPHON=phUS_UpdateChnInfo(ph_config);
// signal mode
	conf_msg.bSURR=cfg_surroundMode?1:0;

// send config message
	host.sendInitCmsg({id:'NEST', mCONF:conf_msg});

// send fader levels
	chnUpdateFaderLevels();

	log.info("Scripts: channel config changed & sent to ACL");
}

function onHostCfgChange()
{
// this function is called when host configuration is changed
	log.info("Scripts: host configuration is changed");
	readConfiguration();
/* TODO: make use of a new configuration */
// load V-Mixer data
	vmixer_loaded=0;
	vmixer_startup();
}

// ------------------------- * SSC source list update handling * -------------------------
var list_update=false;
var new_update=null;

/*
** onSrcListUpdate
*/
function onSrcListUpdate(mask)
{
	var n, tMsk;

	if(!list_update)
	{
		tMsk=mask;
		while(1)
		{
			list_update=true;
		// notify anybody who cares
			if('onSrcLstUpdate' in pages.open) pages.open.onSrcLstUpdate(tMsk);
			if(prod_channel!=null) prodSrcReload(tMsk);
			host.sleep(500); // don't allow to reload list too frequently
			list_update=false;
		// check if there've been other updates during processing
			if(new_update==null) break;
			tMsk=new_update; new_update=null;
		}
	}
	else if(new_update==null)
		new_update=mask; // copy mask
	else
	{
	// merge update masks
		for(n in mask)
			new_update[n]+=mask[n];
	}
}

/*
** onRecvMsg
**
** event on incoming Cmsg
*/
function onRecvMsg(msg)
{
	var subm, n, ind, txt;

	if(msg.id=='HSTR')
	{
		var resp;

	// channel messages
		for(ind in msg)
		{
			if(ind.substr(0,3)=='mCH')
			{
				n=parseInt(ind.substr(3,2), 10);
				if(!(n in ch_data) || !ch_data[n].exists) continue;
				subm=msg[ind];

				if(subm.id=='WRNI')
				{
					if('wFAGN' in subm)
						mod.setChnFader(n, subm.wFAGN/10);
				}
				if(subm.id=='READ')
				{
					if('bFAGN' in subm)
					{
						resp={id:'HSTO'};
						resp[ind]={id:'INDI','wFAGN':clamp(mod.getChnFader(n)*10, -32768, 32767)};
						host.sendRemoteHostCmsg(resp);
					}
					if('bFAMO' in subm)
					{
						resp={id:'NEST'};
						resp[ind]={id:'INDI','bFAMO':ch_data[n].faderStart};
						host.sendRemoteHostCmsg(resp);
					}
					if('bTBCK' in subm)
					{
						resp={id:'NEST'};
						resp[ind]={id:'INDI','bTBCK':ch_data[n].talkback?1:0};
						host.sendRemoteHostCmsg(resp);
					}
				}
			}
			else if(ind=='mSHOW')
			{
				subm=msg[ind];
				if(subm.id=='READ')
				{
					resp={id:'INDI'};
					if('bSHWI' in subm) resp.dSHWI =cur_show_id;
					if('bSHWN' in subm) resp.bvSHWN=strToArr(cur_show_name);
					host.sendRemoteHostCmsg({id:'NEST', mSHOW:resp});
				}
			}
			else if(ind=='mSUPV')
			{
				subm=msg[ind];
				if(subm.id=='READ')
				{
					resp={id:'INDI'};
					if('bSTAT' in subm) resp.bSTAT=eng_status?9:0;
					host.sendRemoteHostCmsg({id:'NEST', mSUPV:resp});
				}
			}
			else if(ind=='mUSIF')
			{
				subm=msg[ind];
				if(subm.id=='READ')
				{
					resp={id:'INDI'};
					if('bPVIN' in subm) resp.bPVIN=pview_interlock?1:0;
					if('bENBL' in subm) resp.dENBL=mstEnable.mask;
					host.sendRemoteHostCmsg({id:'NEST', mUSIF:resp});
				}
			}
		}
		return;
	}
	if(msg.id=="INDI")
	{
		if('wSTMG' in msg) indiMonGain(msg.wSTMG, "mon2");
		if('wPVWG' in msg) indiMonGain(msg.wPVWG, "pvw");
		if('wCRMG' in msg) indiMonGain(msg.wCRMG, "mon1");
		if('wCRHG' in msg) indiMonGain(msg.wCRHG, "hp");

		if('bSCHG' in msg) gui_indi.setShowChanged(msg.bSCHG);
		if('bvSRLU' in msg) onSrcListUpdate(msg.bvSRLU); // source table updated
		if('bTMRT' in msg)
		{
			timer_tenth_mode=msg.bTMRT;
			upTimerObj.updateTMRT();
			dnTimerObj.updateTMRT();
		}
		if('bTMRM' in msg)
		{
			timer_mode=msg.bTMRM;
			lwSendTimerEvent(true, {Mode:timer_mode}); // FIXME: move to timer code
			timer_mode_ind.setIndex(timer_mode);
		}
	}
	else if(msg.id=="NEST")
	{
	// user interface
		if('mUSIF' in msg)
		{
			if('bPVIN' in msg.mUSIF) pview_interlock=(msg.mUSIF.bPVIN==1);
			if('dENBL' in msg.mUSIF) mstEnable.update(msg.mUSIF.dENBL);
		}
	// switchable meters
		if('mMTRX' in msg)
		{
			if(('mPGM3' in msg.mMTRX) && 'wNSEL' in msg.mMTRX.mPGM3)
			{
				mtrxRequestMeter(883, msg.mMTRX.mPGM3.wNSEL);
			// Update Label
				if(msg.mMTRX.mPGM3.wNSEL==803)
					mtr_label[3].setText(null);
				else
					mtr_label[3].setText(mtrxGetDesc(msg.mMTRX.mPGM3.wNSEL));
			}
		}
	// show
		if(('mSHOW' in msg) && msg.mSHOW.id=="INDI")
		{
		// cur show name
			if('bvSHWN' in msg.mSHOW)
			{
				cur_show_name=host.arrtostr(msg.mSHOW.bvSHWN);
				cur_show_loading=false;
				vmixer_startup();
			// update indication on screen
				gui_indi.setShowName(cur_show_name, false);
			// report profile change to LWCP clients
				lwEventProfChange();
			}
		// next show name
			if('bvNXTN' in msg.mSHOW)
			{
				cur_show_name=host.arrtostr(msg.mSHOW.bvNXTN);
				cur_show_loading=true;
				prev_show_id=cur_show_id;
				cur_show_id =msg.mSHOW.dSHWI;
			// update indication on screen
				gui_indi.setShowName(cur_show_name, true);
			// report profile change to LWCP clients
				lwEventProfChange();
			}
		}
		else
		{
			if(('mPREV' in msg) && msg.mPREV.id=='INDI') onPrevIndi(msg.mPREV); // preview
			if(('mSTMO' in msg) && msg.mSTMO.id=='INDI') onMon2Indi(msg.mSTMO); // st monitor (monitor 2)
			if(('mCRMO' in msg) && msg.mCRMO.id=='INDI') onMon1Indi(msg.mCRMO); // cr monitor (monitor 1)
			if(('mCRHP' in msg) && msg.mCRHP.id=='INDI') onHpIndi(msg.mCRHP);   // cr hp monitor (headphones)
		}
		// talk
		if(('mTALK' in msg) && msg.mTALK.id=="INDI")
		{
			if('bTALK' in msg.mTALK) gui_indi.setTalk(msg.mTALK.bTALK);
			if('bSTUD' in msg.mTALK)
			{
				if(msg.mTALK.bSTUD==0)
				{
					mod.setMasterKeyLed(keys.MstTalkToMon2, 0);
					mod.setMasterKeyLed(keys.CmbTalkToMon2, 0);
				}
				else
				{
					mod.setMasterKeyLed(keys.MstTalkToMon2, 255);
					mod.setMasterKeyLed(keys.CmbTalkToMon2, 255);
				}
			}
		}
		else if('mCONF' in msg)
		{
		// config messages
			subm=msg.mCONF;
			if('bNTPS' in subm) ntp_status.setIndex(subm.bNTPS==1?1:0);
		}
		else if(('mMET2' in msg) && ('bvDISP' in msg.mMET2))
		{
		// meter 2 content label
			txt=host.arrtostr(msg.mMET2.bvDISP);
			mtr_label[2].setText((txt=="PGM2")?null:txt);
		}
		else if(('mMET4' in msg) && ('bvDISP' in msg.mMET4))
		{
		// meter 4 content label
			txt=host.arrtostr(msg.mMET4.bvDISP);
			mtr_label[4].setText((txt=="PGM4")?null:txt);
		}
		else if(('mSUPV' in msg) && msg.mSUPV.id=="INDI")
		{
			if('bSTAT' in msg.mSUPV)
			{
				if(msg.mSUPV.bSTAT==9)
				{
				/* Engine Good */
					if(!eng_status)
					{
						eng_status=true;
						updateWarning();
					// V-Mixer
						vmixer_loaded=0;
						vmixer_startup();
					}
				}
				else
				{
				/* Engine Offline */
					if(eng_status)
					{
						eng_status=false;
						updateWarning();
					}
				}
			}
		}
		else if('mPRDL' in msg)
		{
		// profanity delay
			if(msg.mPRDL.id=='INDI' && ('dLPID' in msg.mPRDL) && prdl_lpid!=msg.mPRDL.dLPID)
			{
				prdl_lpid=msg.mPRDL.dLPID;
				if(prdl_lpid==0)
				{
				// No profanity delay
					mod.setMasterKeyLed(keys.MstDump, 0);
					mod.setMasterKeyLed(keys.MstExit, 0);
					mod.setMasterKeyLed(keys.MstPause, 0);
					mod.setMasterKeyLed(keys.MstEngage, 0);
					gui_indi.setDelay(false);
				}
				else
				{
				// TODO: update GPIO status
				}
			}
		}
		else if(('mATMR' in msg) && ('bCMND' in msg.mATMR))
		{
		// commands for up timer
			switch(msg.mATMR.bCMND)
			{
			case timer_cmd.start:
				upTimerObj.start();
				break;
			case timer_cmd.stop:
				upTimerObj.stop();
				break;
			case timer_cmd.reset:
				upTimerObj.reset();
				break;
			case timer_cmd.reset_start:
				upTimerObj.reset();
				upTimerObj.start();
				break;
			}
		}
		else if('mUTMR' in msg)
		{
		// Up timer
			if(msg.mUTMR.id=='INDI' && ('dLPID' in msg.mUTMR) && utmr_lpid!=msg.mUTMR.dLPID)
			{
				utmr_lpid=msg.mUTMR.dLPID;
			// TODO: update
			}
		}
		else if('mDTMR' in msg)
		{
		// Down timer
			if(msg.mDTMR.id=='INDI' && ('dLPID' in msg.mDTMR) && utmr_lpid!=msg.mDTMR.dLPID)
			{
				dtmr_lpid=msg.mDTMR.dLPID;
			// TODO: update
			}
			if('bCMND' in msg.mDTMR)
			{
			// commands for down timer
				switch(msg.mDTMR.bCMND)
				{
				case timer_cmd.reset_start:
					dnTimerObj.reset();
					dnTimerObj.start();
					break;
				}
			}
		}
		else if('mPHON' in msg)
		{
		// phone stuff
			phUS_UpdateShowInfo(msg.mPHON);
		}
		else if('mRCMO' in msg)
		{
		// Record Mode stuff
			subm=msg.mRCMO;
			if(subm.id=='WRIN')
			{
				if('bENBL' in subm) recmode_enabled=(subm.bENBL!=0);
				if(('mRCDR' in subm) && ('dLPID' in subm.mRCDR)) recmode_rcdrlpid=subm.mRCDR.dLPID;
			}
			else if(subm.id=='INDI')
			{
				if('bSTAT' in subm)
				{
					if(subm.bSTAT==0)
						finishDeactivatingRecordMode();
					else
						finishActivatingRecordMode();
				}
			}
		}
		else
		{
		// channel messages
			for(ind in msg)
			{
				if(ind.substr(0,3)=='mCH' ) onChnRecvMsg(parseInt(ind.substr(3,2), 10), msg[ind]);
				if(ind.substr(0,4)=='mRET') onRetRecvMsg(parseInt(ind.substr(4,1), 10), msg[ind]);
			}
		}
		if('mVMIX' in msg)
			vmixer_process_msg(msg.mVMIX);
	}

// for open pages call INDI handlers
	for(ind in pages.rxHandlersGlobal)
		pages.rxHandlersGlobal[ind](msg); //call function
}

/*
** sendGPIOcommand
**
** sends GPIO command (uses ACL for now)
*/
function sendGPIOcommand(lpid, circuit, value)
{
	if(lpid==0) return; // lpid of 0 means logic disabled
	host.sendNestCmsg('mGPIO', {id:'WRNI', dLPID:lpid, bvPINN:[circuit], bvVALL:[value]});
}

/*
** onGPIOevent
**
** process incoming GPIO event
*/
function onGPIOevent(lpid, circuit, value)
{
	var n;

// Channel GPIO
	for(n=1; n<=64; n++)
	{
		if(ch_data[n].exists && ch_data[n].lpid==lpid)
			onChnGPIOevent(n, circuit, value);
	}
// CR Monitor
	if(lpid==crmo_lpid)
	{
		switch(circuit)
		{
		case gpioCirc.muteCR:
			host.sendNestCmsg('mCRMO', {id:'WRIN', bMUCM:(value?1:0)});
			break;
		case gpioCirc.dimCR:
			host.sendNestCmsg('mCRMO', {id:'WRIN', bDIMC:(value?1:0)});
			break;
		case gpioCirc.extPreview:
			host.sendNestCmsg('mPREV', {id:'WRIN', bEXON:(value?1:0)});
			break;
		case gpioCirc.talkToExt:
			host.sendNestCmsg('mTALK', {id:'WRIN', bEXTR:(value?1:0)});
			break;
		}
	}
// ST Monitor
	if(lpid==stmo_lpid)
	{
		switch(circuit)
		{
		case gpioCirc.muteST:
			host.sendNestCmsg('mSTMO', {id:'WRIN', bMUCM:(value?1:0)});
			break;
		case gpioCirc.dimST:
			host.sendNestCmsg('mSTMO', {id:'WRIN', bDIMC:(value?1:0)});
			break;
		case gpioCirc.timer:
			if(value)
			{
				upTimerObj.reset();
				upTimerObj.start();
			}
			break;
		case gpioCirc.countdown:
			if(value)
			{
				dnTimerObj.reset();
				dnTimerObj.start();
			}
			break;
		}
	}
// Profanity Delay
	if(lpid==prdl_lpid)
	{
		switch(circuit)
		{
		case gpioCirc.delayDump:
			mod.setMasterKeyLed(keys.MstDump, value?255:0);
			break;
		case gpioCirc.delayExit:
			mod.setMasterKeyLed(keys.MstExit, value?255:0);
			break;
		case gpioCirc.delayPause:
			mod.setMasterKeyLed(keys.MstPause, value?255:0);
			break;
		case gpioCirc.delayEngage:
			mod.setMasterKeyLed(keys.MstEngage, value?255:0);
			break;
		case gpioCirc.delayAnnounce:
			gui_indi.setDelay(value);
			break;
		}
	}
// Up Timer
	if(lpid==utmr_lpid)
	{
		switch(circuit)
		{
		case gpioCirc.timerStart:
			if(value) upTimerObj.start();
			break;
		case gpioCirc.timerStop:
			if(value) upTimerObj.stop();
			break;
		case gpioCirc.timerReset:
			if(value) upTimerObj.reset();
			break;
		case gpioCirc.timerFreeze:
			if(value)
				upTimerObj.freeze();
			else
				upTimerObj.unfreeze();
			break;
		}
	}
// Down Timer
	if(lpid==dtmr_lpid)
	{
		switch(circuit)
		{
		case gpioCirc.timerStart:
			if(value) dnTimerObj.start();
			break;
		case gpioCirc.timerStop:
			if(value) dnTimerObj.stop();
			break;
		case gpioCirc.timerReset:
			if(value) dnTimerObj.reset();
			break;
		case gpioCirc.timerFreeze:
			if(value)
				dnTimerObj.freeze();
			else
				dnTimerObj.unfreeze();
			break;
		}
	}
}

function updateWarning()
{
	switch(net_status)
	{
	case 0:
	default:
		net_status=0;
		pages.main.showWarning("ERROR: NO NETWORK CONNECTION");
		break;
	case 1:
		if(eng_status)
			pages.main.hideWarning();
		else
			pages.main.showWarning("ERROR: NO CONNECTION TO ENGINE");
		break;
	case 2:
		if(eng_status)
			pages.main.showWarning("WARNING: DUPLEX MISMATCH (CHECK NETWORK SETUP)");
		else
			pages.main.showWarning("ERROR: NO CONNECTION TO ENGINE");
		break;
	}
}

function onHostNetChange(status)
{
	if(net_status==status) return;

	net_status=status;
	updateWarning();
}

// -------------------- page functions ------------------------
function createPages()
{
	showPageCreate();
	sendsPageCreate();
	returnsPageCreate();
	monsrcPageCreate();
	ipPageCreate();

	timerPageCreate();
	monoptPageCreate();
	monopt2PageCreate();
	hpeqPageCreate();
	metersPageCreate();
	mtrxPageCreate();
	helpPageCreate();

	choptPageCreate();
	chsrcPageCreate();
	cheqPageCreate();
	chmicprocPageCreate();
	chpanPageCreate();
	chauxPageCreate();
	chm1PageCreate();

// main page should be created last one, so that it shows up on screen
	mainPageCreate();

// create control section
	Gui_CreateControlSection(pages.main);

	sknobsHide();
	onOpenPage(pages.main, null);
}

// opens, hides of updates page data
// channel_nr - optional for channel option pages
function activatePage(a_page, channel_nr)
{
	if(pages.open!=a_page)
	{
	// close other page && display this
		onClosePage(pages.open);
		onOpenPage(a_page, channel_nr);
		pages.open.frame.hide();
		a_page.frame.show();
		pages.open=a_page;
		return;
	}
// same page: jump to other channel or close page
	if(('channel' in pages.open) && pages.open.channel!=channel_nr)
	{
	// update page with new channel information
		onClosePage(pages.open);
		onOpenPage(a_page, channel_nr);
	}
	else
	{
	// close this page && return to main
		onClosePage(pages.open);
		onOpenPage(pages.main, null);
		pages.open.frame.hide();
		pages.main.frame.show();
		pages.open=pages.main;
	}
}
function closePage(a_page, channel_nr)
{
	if(pages.open!=a_page) return;
	if(('channel' in pages.open) && pages.open.channel!=channel_nr) return;

// close this page && return to main
	onClosePage(pages.open);
	onOpenPage(pages.main, null);
	pages.open.frame.hide();
	pages.main.frame.show();
	pages.open = pages.main;
}
// if some channel options page is open, switch to different suboption page
// for the same channel
function chSwitchToPage(a_page)
{
// check if current page is channel page
	if(!('channel' in pages.open) || pages.open.channel<1 || pages.open.channel>64) return;

// Check lock map
	switch(a_page)
	{
	case pages.chsrc:     if(!mstEnable.chn_src ) return; break;
	case pages.chm1:      if(!mstEnable.chn_feed) return; break;
	case pages.cheq:      if(!mstEnable.chn_eq  ) return; break;
	case pages.chmicproc: if(!mstEnable.chn_eq  ) return; break;
	case pages.chpan:     if(!mstEnable.chn_pan ) return; break;
	case pages.chaux:     if(!mstEnable.chn_aux ) return; break;
	}

// if already open, close and go back to default option view page
	if(pages.open==a_page) a_page=pages.chopt;

	onClosePage(pages.open);
	onOpenPage(a_page, pages.open.channel);
	pages.open.frame.hide();
	a_page.frame.show();
	pages.open=a_page;
}
// if some channel opt page is open, switch to different channel, but keep the same suboption page
function chSwitchToChannel(channel_nr)
{
	if('channel' in pages.open)
	{
		if(pages.open==pages.chmicproc && (ch_data[channel_nr].srct==srct.line || ch_data[channel_nr].srct==srct.computer))
		{
		// workaround: close mic-processing page if src_type is line/computer
			onClosePage(pages.open); // close micprocessing page - old channel
			onOpenPage(pages.cheq, channel_nr); // open EQ page - new channel
			pages.open.frame.hide();
			pages.cheq.frame.show();
			pages.open=pages.cheq;
		}
		else
		{
			onClosePage(pages.open);
			onOpenPage(pages.open, channel_nr);
		}
	}
}

// ------------------------- * Page events * -------------------------

// update page with data from ACL. Turn on lamp on module if necessary
function onOpenPage(a_page, channel_nr)
{
	if('onOpen' in a_page) a_page.onOpen(channel_nr);
// let production code to decide if he needs this page
	prodEnable(channel_nr);
}
// turn off lamp on module
function onClosePage(a_page)
{
	if('onClose' in a_page) a_page.onClose(a_page);
}
function onNumpadDown(num)
{
	if(!('onNumpadDown' in pages.open)) return false;
	pages.open.onNumpadDown(num);
	return true;
}
